# GUI应用程序主程序
import sys

from PyQt5.QtCore import QTranslator
from PyQt5.QtGui import QIcon
from PyQt5.QtNetwork import QLocalSocket, QLocalServer
from PyQt5.QtWidgets import QApplication

from src.config import configure
from src.const.const import sys_const
from src.controller.myFrame import QmyFrame
from src.exception.MyException import myExcepthook
from src.lib.MyQApplication import QSingleApplication
from src.lib.gifSplashScreen import GifSplashScreen
from src.lib.myFramelessQWidget import FramelessWindow
from src.module import logHelper

# 样式
StyleSheet = """
/*标题栏*/
TitleBar {
    background-color: #efefef;
}
/*最小化最大化关闭按钮通用默认背景*/
#buttonMinimum,#buttonMaximum,#buttonClose {
    border: none;
    background-color: #efefef;
}
/*悬停*/
#buttonMinimum:hover,#buttonMaximum:hover {
    background-color: white;
    color: red;
}
#buttonClose:hover {
    color: red;
}
/*鼠标按下不放*/
#buttonMinimum:pressed,#buttonMaximum:pressed {
    background-color: #ffc1c1;
}
#buttonClose:pressed {
    color: red;
    background-color: #ffc1c1;
}

"""

sys.excepthook = myExcepthook
def main():
    # app = QApplication(sys.argv)  # 创建GUI应用程序

    app = QSingleApplication(sys.argv) # 创建GUI单例应用程序

    if app.isRunning():
        app.sendMessage("app is running")
        sys.exit(0)

    # 处理其他
    logs = logHelper.logger.getLogger(__name__)
    logs.info("启动应用程序")

    app.setStyleSheet(StyleSheet)

    # 汉化
    translator = QTranslator()
    translator.load('./i18n/qm/qt_zh_CN.qm')
    app.installTranslator(translator)
    splash = GifSplashScreen('start_image.png')
    splash.show()
    mainWnd = FramelessWindow()
    app.setActivationWindow(mainWnd)
    mainWnd.setWindowTitle(configure.title)
    mainWnd.setWindowIcon(QIcon(sys_const.WEATHER_LOGO_PATH))
    mainWnd.setWidget(QmyFrame(mainWnd))  # 把自己的窗口添加进来

    # 重新设置大小
    desktop = QApplication.desktop()  # 取得显示器对象
    screenRect = desktop.screenGeometry()  # 取得屏幕大小
    height = screenRect.height() / 2  # 取屏幕高的一半作为高
    width = screenRect.width() / 2  # 取屏幕宽的一半作为宽
    mainWnd.resize(width, height)  # 重新设置大小

    mainWnd.show()
    app.processEvents()
    splash.close()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()
