#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   const.py    
@Contact :   wangxin01@loongson.cn
@License :   (C)Copyright 2021

@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
21-5-25 上午1:47   wangxin    1.0         None
'''


class Const:
    """
    自定义常量：（1）命名全部大写；（2）值不可修改
    """

    class ConstError(TypeError):
        pass

    class ConstCaseError(ConstError):
        pass

    def __setattr__(self, name, value):
        if name in self.__dict__.keys():
            raise self.ConstError('Can not change const.{0}'.format(name))
        if not name.isupper():
            raise self.ConstCaseError(
                'const name {0} is not all uppercase.'.format(name))
        self.__dict__[name] = value


# 系统常量
sys_const = Const()

# 图片路径
sys_const.IMAGE_PATH = "./images/"

# 天气类型图片路径
sys_const.IMAGE_PATH_WEATHER = "./images/weather/"

# 天气图标路径
sys_const.WEATHER_ICON_PATH = "./images/color-64/"
sys_const.LOGO_PATH  = "./images/dark.png"
sys_const.WEATHER_LOGO_PATH  = "./images/weather_logo.svg"
# PNG图片后缀
sys_const.PNG_SUFFIX = ".png"

# JPG图片后缀
sys_const.JPG_SUFFIX = ".JPG"

# 时间格式化字符串 HH:MM
sys_const.TIME_FORMATTER_HH_MM = "%H:%M"

# 时间格式化字符串 HH
sys_const.TIME_FORMATTER_HH = "%H"

# UTC格式
sys_const.UTC_FORMATTER = "%Y-%m-%dT%H:%M+08:00"

# 时间格式 yyyy-mm-dd
sys_const.TIME_FORMATTER_YYYY_DD_MM_HYPHEN = "%Y-%m-%d"

# 时间格式 mm月dd日
sys_const.TIME_FORMATTER_DD_MM_HYPHEN_CHAR = "%m月%d日"

# 时间格式化字符串 dd日
sys_const.TIME_FORMATTER_DD = "%d日"

# 时间格式化字符串 周数
sys_const.TIME_FORMATTER_W = "%w"

# 摄氏度符号 °C
sys_const.TEMPERATURE_MARK_C = "°C"

# 摄氏度省略符号 °
sys_const.TEMPERATURE_MARK_C_THUMB = "°"

# 汉字 级
sys_const.CHINESE_LEVEL = "级"

# 汉字 市
sys_const.CHINESE_CITY = "市"

# 汉字 省
sys_const.CHINESE_PROVINCE = "省"

# 左括号
sys_const.BRACKET_LEFT = "("

# 右括号
sys_const.BRACKET_RIGHT = ")"

# 波浪符号
sys_const.TILDE_MARK = "~"

# 配置文件路径
sys_const.CONF_PATH = "./conf/config.yaml"

# 城市列表CSV文件路径
sys_const.LOCATION_LIST_CSV_PATH = "./db/China-City-List-latest.csv"

# SQLite数据库路径
sys_const.SQLITE_DB_PATH = "./db/app.db"

# encoding utf-8
sys_const.UTF_8 = "utf-8"

# 百分比符号 %
sys_const.PERCENT_MARK = "%"

# 风速单位 千米每小时
sys_const.WIND_SPEED_UNIT = "千米/小时"

# 空格
sys_const.SPACE = " "

# 逗号
sys_const.COMMA = ","

# 返回状态码 200
sys_const.RESULT_CODE_200 = "200"

# 返回状态码 0
sys_const.RESULT_CODE_0 = 0

# 空字符串
sys_const.EMPTY = ""

# log配置文件路径
sys_const.LOG_CONF_PATH = "./conf/log.conf"

# 日志级别 error
sys_const.LOG_ERROR = "error"

# 日志级别 info
sys_const.LOG_INFO = "info"

# 星期一，二，三，四，五，六，日
sys_const.WEEKLY_CHARS = "一二三四五六日"

# 星期汉字
sys_const.WEEKLY_CN = "星期"


# 配置文件常量
conf_const = Const()

# 应用程序设置
conf_const.APLLCATION_SETTINGS = "apllcation_settings"

# 应用程序名称
conf_const.TITLE = "title"

# 收藏列表限制个数
conf_const.LOCATION_COLLECTION_LIMIT_NUM = "location_collection_limit_num"

# 错误页面标题
conf_const.ERROR_PAGE_TITLE = "error_page_title"

# 问题页面标题
conf_const.QUESTION_PAGE_TITLE = "question_page_title"

# API URL
conf_const.API_URL = "api_url"

# 天气API URL
conf_const.WEATHER_API_URL = "weather_api_url"

# API KEY
conf_const.API_KEY = "api_key"

# 实时天气信息API
conf_const.CURRENT_WEATHER_DATA_API = "current_weather_data_api"

# 实时空气质量API
conf_const.CURRENT_AIR_QUALITY_DATA_API = "current_air_quality_data_api"

# 实逐小时天气API
conf_const.GET_DAILY_WEATHER_DATA_API = "get_daily_weather_data_api"

# 实逐小时天气API
conf_const.GET_HOURLY_WEATHER_DATA_API = "get_hourly_weather_data_api"

# 获取城市信息通过城市名称
conf_const.GET_LOCATION_INFO_API = "get_location_info_api"

# 位置信息API
conf_const.LOCATION_API_URL = "location_api_url"

# 根据IP地址获取城市信息API
conf_const.LOCATION_API = "location_api"

#关于
conf_const.ABOUT = "about"
conf_const.VERSION = "version"
# 数据提供方
conf_const.PROVIDER = "provider"
conf_const.IMAGE = "image"
conf_const.LOGO = "logo"

#定时刷新间隔时间
conf_const.TIMED_REFRESH = "timed-refresh"
conf_const.SETTING_PARAMETER = "setting_parameter"

conf_const.CURRENT_LOCATION = "currentLocation"


conf_const.THEME_SETTING_STUTE= "theme_setting_stute"


conf_const.THEME_SETTING = "theme_setting"
conf_const.OTHER_PAGE = "other_page"
conf_const.INCLUDE_PAGE = "include_page"
conf_const.LIGHTTHEME = "lightTheme"
conf_const.DARKTHEME = "darkTheme"
conf_const.DEFULTTHEME = "defultTheme"

