#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   myCollectedPage.py    
@Contact :   wangxin01@loongson.cn
@License :   (C)Copyright 2021

@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
21-6-1 上午2:32   wangxin    1.0         None
'''

# import lib
import yaml
from PyQt5.QtCore import pyqtSignal, QThread
from PyQt5.QtWidgets import (
    QApplication, QWidget)
from yaml import SafeLoader

from src.config import configure
from src.config.configure import currentLocation
from src.const.const import sys_const, conf_const
from src.const.styleSheet import CollectStyleSheet, styleSheetTransparent, fontColorWhite
from src.controller.myCollectionItemPage import QmyCollectionItemPage
from src.controller.myMessageBox import QmyMessageBox
from src.lib.flowLayout import FlowLayout
from src.lib.gifSplashScreen import GifSplashScreen
from src.message.message import Q0002
from src.model.location import Location
from src.services import myServices
from src.ui.collectedPage import Ui_CollectedPage


# 取得收藏地区列表
def getLocationCollectionItems():
    location_datas = myServices.get_location_collection_datas(configure.location_collection_limit_num)
    datas = list()
    if location_datas:
        for location in location_datas:
            collection = getCollectionItem(location)
            datas.append(collection)
            QApplication.processEvents()
    return datas


# 取得收藏地区
def getCollectionItem(location):
    locationCollection = Location()
    locationCollection.id = location.id
    locationCollection.city = location.city
    locationCollection.deleted_flag = location.deleted_flag
    locationCollection.district = location.district
    locationCollection.ip = location.ip
    locationCollection.lat = location.lat
    locationCollection.lng = location.lng
    locationCollection.nation = location.nation
    locationCollection.province = location.province
    weather = myServices.get_current_weather_data(location)

    locationCollection.temperature = weather.get("now").get("temp") + sys_const.TEMPERATURE_MARK_C  # 该时间点的温度
    locationCollection.weather = weather.get("now").get("text")  # 天气
    wind_info_str = weather.get("now").get("windDir") + " " + weather.get(
        "now").get(
        "windScale") + sys_const.CHINESE_LEVEL
    locationCollection.wind_info = wind_info_str  # 风速

    daily_datas = myServices.get_daily_weather_data(location)  # 取得周天气预报信息

    locationCollection.temperature_max = daily_datas.get("daily")[0].get(
        "tempMax") + sys_const.TEMPERATURE_MARK_C_THUMB  # 最高温度
    locationCollection.temperature_min = daily_datas.get("daily")[0].get(
        "tempMin") + sys_const.TEMPERATURE_MARK_C_THUMB  # 最低温度
    return locationCollection


# 通过ip地址取得当前城市信息API
def get_location_by_ip():
    # strpar = open('./conf/parameter', encoding="utf-8").read()
    location = Location()
    with \
            open(sys_const.CONF_PATH, encoding=sys_const.UTF_8) as file_obj:
        setting_parameter_data = yaml.load(file_obj, SafeLoader)
        setting_parameter = setting_parameter_data.get(conf_const.APLLCATION_SETTINGS).get(conf_const.SETTING_PARAMETER)
    if str(setting_parameter) == str(currentLocation):
        data = myServices.get_location_by_ip(None)
        location.ip = data.get("result").get("ip")  # IP地址
        location.lat = data.get("result").get("location").get("lat")  # 纬度
        location.lng = data.get("result").get("location").get("lng")  # 经度
        location.nation = data.get("result").get("ad_info").get("nation")  # 国家地区
        location.province = data.get("result").get("ad_info").get("province")  # 省份
        location.city = data.get("result").get("ad_info").get("city")  # 城市
        location.name = data.get("result").get("ad_info").get("city")  # 城市
        location.district = data.get("result").get("ad_info").get("district")  # 区
        location.id = myServices.get_location_id_by_location_name(location.city)  # 地区ID
    else:
        data = myServices.get_location_id_by_location_name(str(setting_parameter))
        location.id = data.location_id
        location.province = data.adm1
        location.nation = data.country_name
        location.city = data.adm2
        location.name = data.adm2
        location.lng = data.longitude
        location.lat = data.latitude
    return location


class QmyCollectedPage(QWidget):
    # 自定义信号
    itemClicked = pyqtSignal(object)

    def __init__(self, parent=None):
        self.imageheight = 100
        super().__init__(parent)  # 调用父类构造函数，创建QWidget窗体
        self.__ui = Ui_CollectedPage()  # 创建UI对象
        self.__ui.setupUi(self)  # 构造UI

        self.flowLayout = FlowLayout()  # 创建显示收藏列表的瀑布流组件

        self.setStyleSheet(styleSheetTransparent + fontColorWhite)
        self.setSettingFrameData()

        self.collectionData = CollectionData()  # 异步加载数据
        self.collectionData.signal.connect(self.callback)
        self.collectionData.start()  # 启动线程

    def setSettingFrameData(self):
        themecollection = getCollectionItem(get_location_by_ip())
        widget = QmyCollectionItemPage()
        widget.setPageDatas(themecollection)
        widget.clicked.connect(self.itemOnClicked)
        self.flowLayout1 = FlowLayout()
        self.__ui.themeframe.setLayout(self.flowLayout1)
        self.flowLayout1.addWidget(widget)
        self.__ui.themeframe.resize(416, 140)
        widget.setStyleSheet(CollectStyleSheet.collectItemHover + CollectStyleSheet.stylesheetItemCollect)

    # 设置页面点击主题设置时的槽函数
    def locationItemOnClicked(self):
        self.mainPage.location = get_location_by_ip()
        self.mainPage.setWeatherDatas()
        self._location = self.mainPage.location
        self.setCollectButoonStyle()

    def callback(self, signal):
        self.locationCollectionItems = signal
        for item in iter(signal):
            QApplication.instance().processEvents()
            widget = QmyCollectionItemPage()
            widget.setMinimumWidth(250)
            widget.resize(416, 140)
            widget.setPageDatas(item)
            widget.clicked.connect(self.itemOnClicked)
            widget.rightMenuOnClick.connect(self.rightMenuOnClick)
            # 设置背景选中以及正常态透明度
            widget.setStyleSheet(CollectStyleSheet.collectItemHover + CollectStyleSheet.stylesheetItemCollect)
            # "#backGroundFrame{border-image:url(%sbackground.jpg)}" % sys_const.IMAGE_PATH)

            self.flowLayout.addWidget(widget)

        self.__ui.frame.setLayout(self.flowLayout)
        self.setWindowTitle("Flow Layout")
        # splash.finish(self)

    # 点击一个城市的处理事件
    def itemOnClicked(self, e):
        self.itemClicked.emit(e.item)

    # 右键菜单
    def rightMenuOnClick(self, e):
        result = QmyMessageBox.question(self, configure.question_page_title, Q0002,
                                        QmyMessageBox.YES | QmyMessageBox.NO)
        if result == 1:
            for i in range(len(self.locationCollectionItems)):
                if e.item.id == self.locationCollectionItems[i].id:
                    item = self.flowLayout.itemAt(i)

                    location = Location()
                    location.id = self.locationCollectionItems[i].id
                    location.city = self.locationCollectionItems[i].display_name
                    location.deleted_flag = True
                    myServices.upsert_location_collect(location)
                    self.flowLayout.removeItem(item)
                    item.widget().deleteLater()
                    del self.locationCollectionItems[i]



class CollectionData(QThread):
    signal = pyqtSignal(object)  # 括号里填写信号传递的参数

    def __init__(self):
        super().__init__()

    def __del__(self):
        self.wait()

    def run(self):
        # 进行任务操作
        global splash
        splash = GifSplashScreen('spinner.gif')
        splash.show()
        try:
            locationCollectionItems = getLocationCollectionItems()
            splash.close()
        except:
            splash.close()
        self.signal.emit(locationCollectionItems)  # 发射信号


if __name__ == '__main__':
    import sys

    app = QApplication(sys.argv)
    mainWin = QmyCollectedPage()
    mainWin.show()
    sys.exit(app.exec_())
