#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   myFrame.py    
@Contact :   wangxin01@loongson.cn
@License :   (C)Copyright 2021

@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
21-5-31 上午7:55   wangxin    1.0         None
'''

# import lib
import traceback
from collections import OrderedDict

import yaml
from PyQt5 import QtCore
from PyQt5.QtCore import pyqtSignal, Qt, QThread, QTimer, QStringListModel, pyqtSlot
from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtWidgets import QWidget, QApplication, QCompleter
from yaml import SafeLoader

from src.config import configure
from src.config.configure import timer, currentLocation
from src.const.const import sys_const, conf_const
from src.const.styleSheet import FrameStyleSheet, styleSheetTransparent20, menuWidgetStyleSheet
from src.const.weatherImage import cloudy_strings, cloudy_image, rainy_strings, rainy_image, snow_strings, snow_image, \
    sunny_image, overcast_image, overcast_strings
from src.controller.myCollectedPage import QmyCollectedPage
from src.controller.myMainPage import QmyMainPage
from src.controller.myMessageBox import QmyMessageBox
from src.controller.settingsPage import setPage
from src.exception.BusinessException import BusinessException
from src.message.message import Q0001, Q0002, E0002, E0003, E0001
from src.model.location import Location
from src.model.myEnum import MenuEnum
from src.module import logHelper
from src.services import myServices
from src.ui.Frame import Ui_Frame
from src.ui.myMenuListItem import QmyMenuListItem
from src.utils.stringUtils import isEmpty

logs = logHelper.logger.getLogger(__name__)


# 通过ip地址取得当前城市信息API
def get_location_by_ip():
    location = Location()
    with \
            open(sys_const.CONF_PATH, encoding=sys_const.UTF_8) as file_obj:
        setting_parameter_data = yaml.load(file_obj, SafeLoader)
        setting_parameter = setting_parameter_data.get(conf_const.APLLCATION_SETTINGS).get(conf_const.SETTING_PARAMETER)
    if str(setting_parameter) == str(currentLocation):
        data = myServices.get_location_by_ip(None)
        location.ip = data.get("result").get("ip")  # IP地址
        location.lat = data.get("result").get("location").get("lat")  # 纬度
        location.lng = data.get("result").get("location").get("lng")  # 经度
        location.nation = data.get("result").get("ad_info").get("nation")  # 国家地区
        location.province = data.get("result").get("ad_info").get("province")  # 省份
        location.city = data.get("result").get("ad_info").get("city")  # 城市
        location.name = data.get("result").get("ad_info").get("city")  # 城市
        location.district = data.get("result").get("ad_info").get("district")  # 区
        location.id = myServices.get_location_id_by_location_name(location.city)  # 地区ID
    else:
        data = myServices.get_location_id_by_location_name(str(setting_parameter))
        location.id = data.location_id
        location.province = data.adm1
        location.nation = data.country_name
        location.city = data.adm2
        location.name = data.adm2
        location.lng = data.longitude
        location.lat = data.latitude
    return location


class QmyFrame(QWidget):
    # 自定义信号
    mySignal = pyqtSignal(str)

    def __init__(self, parent=None):
        super().__init__(parent)  # 调用父类构造函数，创建QWidget窗体
        self.menueIsDisplay = False
        self.__ui = Ui_Frame()  # 创建UI对象
        self.__ui.setupUi(self)  # 构造UI

        # 重新设置大小
        self.desktop = QApplication.desktop()  # 取得显示器对象
        self.screenRect = self.desktop.screenGeometry()  # 取得屏幕大小
        self.height = self.screenRect.height() / 2  # 取屏幕高的一半作为高
        self.width = self.screenRect.width() / 2  # 取屏幕宽的一半作为宽
        self.resize(self.width, self.height)  # 重新设置大小

        self.__ui.search_textbox.returnPressed.connect(self.searchTextBoxOnEnter)
        self._location = Location()
        self.mainPage = QmyMainPage()
        self.mainPage.setStyleSheet(FrameStyleSheet.styleSheetTransparent)
        self._location = Location()
        self.__ui.stackedWidget.addWidget(self.mainPage)
        self.__ui.search_textbox.setClearButtonEnabled(True)
        self.__ui.search_textbox.textEdited.connect(self.searchTextOnChanged)  # 绑定槽函数
        self.__ui.search_textbox.setContextMenuPolicy(QtCore.Qt.ActionsContextMenu)
        self.__ui.search_textbox.setPlaceholderText("搜索")
        # 设置自动补全
        self.m_model = QStringListModel()
        self.m_completer = QCompleter(self.m_model, self)
        self.m_completer.setCompletionMode(QCompleter.UnfilteredPopupCompletion)
        self.m_completer.popup().setMinimumHeight(200)
        self.__ui.search_textbox.setCompleter(self.m_completer)
        self.m_completer.activated[str].connect(self.onTxtChoosed)

        self.isConnect = True
        self.menuIsDisplay()
        try:
            self.mainPage.location = get_location_by_ip()
            self.mainPage.setWeatherDatas()
            self.__ui.stackedWidget.setCurrentWidget(self.mainPage)
            self._location = self.mainPage.location
        except BusinessException as be:
            self.setErrorPage()
            self.isConnect = False
            QmyMessageBox.error(self, configure.error_page_title, be.msg, QmyMessageBox.OK)
        except BaseException as e:
            self.setErrorPage()
            self.isConnect = False
            QmyMessageBox.error(self, configure.error_page_title, str(e), QmyMessageBox.OK)

        self.settingsPage = QWidget()
        self.__ui.stackedWidget.addWidget(self.settingsPage)

        self.setPageStyle()  # 设置式样

        self.setMenu()  # 设置菜单列表

        self.search_data = OrderedDict()

        self.setBackgroundImage(self.mainPage.weatherInfo)

    # 设置画面式样
    def setPageStyle(self):
        # 无边框
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.__ui.backgroundframemain.setStyleSheet(
            "#backgroundframemain{border-image:url(%ssunny_background.jpg)}" % sys_const.IMAGE_PATH_WEATHER)  # 设置背景图片
        self.__ui.frame.setStyleSheet(styleSheetTransparent20)  # 标题设为20%透明度
        self.__ui.menu_list.setStyleSheet(FrameStyleSheet.stylesheet)  # 菜单设为20%透明度
        self.__ui.search_btn.setStyleSheet(FrameStyleSheet.stylesheetQPushButton)  # 搜索按钮美化
        self.__ui.refresh_btn.setStyleSheet(FrameStyleSheet.stylesheetQPushButton)  # 刷新按钮美化
        self.__ui.menu_btn.setStyleSheet(FrameStyleSheet.stylesheetQPushButton)  # 菜单按钮美化
        self.__ui.collect_btn.setStyleSheet(FrameStyleSheet.stylesheetQPushButton)  # 收藏按钮美化

        # self.__ui.frame.setWindowOpacity(0.5)
        self.setWindowTitle(configure.configure_data
                            .get(conf_const.APLLCATION_SETTINGS)
                            .get(conf_const.TITLE))  # 从配置文件取得应用名字，设置应用程序标题
        self.setWindowIcon(QIcon(sys_const.IMAGE_PATH + "weather_logo.svg"))  # 设置应用程序logo

        self.__ui.menu_btn.setIcon(QIcon(sys_const.IMAGE_PATH + "menu_btn_icon.svg"))  # 设置菜单按钮的图标
        self.__ui.search_btn.setIcon(QIcon(sys_const.IMAGE_PATH + "search_btn_icon.svg"))  # 设置搜索按钮的图标
        self.__ui.refresh_btn.setIcon(QIcon(sys_const.IMAGE_PATH + "refresh_btn_icon.svg"))  # 设置刷新按钮的图标

        self.setCollectButoonStyle()  # 设置收藏按钮式样

    # 根据天气设置背景
    def setBackgroundImage(self, weatherInfo):
        # 是否是多云
        for text in cloudy_strings:
            if text in weatherInfo:
                self.__ui.backgroundframemain.setStyleSheet(
                    "#backgroundframemain{border-image:url(%s%s)}" % (
                        sys_const.IMAGE_PATH_WEATHER, cloudy_image))  # 设置背景图片
                return
        # 是否雨天
        for text in rainy_strings:
            if text in weatherInfo:
                self.__ui.backgroundframemain.setStyleSheet(
                    "#backgroundframemain{border-image:url(%s%s)}" % (
                        sys_const.IMAGE_PATH_WEATHER, rainy_image))  # 设置背景图片
                return
        # 是否雪天
        for text in snow_strings:
            if text in weatherInfo:
                self.__ui.backgroundframemain.setStyleSheet(
                    "#backgroundframemain{border-image:url(%s%s)}" % (sys_const.IMAGE_PATH_WEATHER,
                                                                      snow_image))  # 设置背景图片
                return
        # 是否阴天
        for text in overcast_strings:
            if text in weatherInfo:
                self.__ui.backgroundframemain.setStyleSheet(
                    "#backgroundframemain{border-image:url(%s%s)}" % (sys_const.IMAGE_PATH_WEATHER,
                                                                      overcast_image))  # 设置背景图片
                return
        # 默认晴天图片
        self.__ui.backgroundframemain.setStyleSheet(
            "#backgroundframemain{border-image:url(%s%s)}" % (sys_const.IMAGE_PATH_WEATHER,
                                                              sunny_image))  # 设置背景图片

    # 设置收藏按钮式样
    def setCollectButoonStyle(self):
        if self.isConnect:
            self.__ui.collect_btn.show()
            self.__ui.collect_btn.setEnabled(True)
            if myServices.location_collect_exists(self._location):
                if myServices.get_location_collection_data(self._location.id).deleted_flag:
                    self.__ui.collect_btn.setIcon(QIcon(sys_const.IMAGE_PATH + "collect_btn_icon.svg"))
                else:
                    self.__ui.collect_btn.setIcon(
                        QIcon(QPixmap(sys_const.IMAGE_PATH + "collected_btn_icon.svg").scaled(40, 40)))
            else:
                self.__ui.collect_btn.setIcon(QIcon(sys_const.IMAGE_PATH + "collect_btn_icon.svg"))
        else:
            self.__ui.collect_btn.hide()
            self.__ui.collect_btn.setEnabled(False)

    # 设置菜单列表
    def setMenu(self):
        menuItem = QmyMenuListItem(sys_const.IMAGE_PATH + "menu_home_icon.svg", "预报", MenuEnum.report)
        menuItem2 = QmyMenuListItem(sys_const.IMAGE_PATH + "menu_collection_icon.svg", "收藏", MenuEnum.collection)
        menuItem3 = QmyMenuListItem(sys_const.IMAGE_PATH + "menu_settings_icon.png", "设置", MenuEnum.settings)

        self.__ui.menu_list.addItem(menuItem)
        self.__ui.menu_list.setItemWidget(menuItem, menuItem.widget)

        self.__ui.menu_list.addItem(menuItem2)
        self.__ui.menu_list.setItemWidget(menuItem2, menuItem2.widget)

        self.__ui.menu_list.addItem(menuItem3)
        self.__ui.menu_list.setItemWidget(menuItem3, menuItem3.widget)

        self.__ui.menu_list.setCurrentRow(0)
        self.__ui.menu_list.setStyleSheet(menuWidgetStyleSheet)

    # 显示菜单
    def menuIsDisplay(self):
        if not self.menueIsDisplay:
            self.__ui.menu_panel.setMaximumWidth(70)
            self.menueIsDisplay = True
        else:
            self.__ui.menu_panel.setMaximumWidth(220)
            self.menueIsDisplay = False

    # 刷新按钮功能
    def refreshButtonOnClicked(self):
        self.reloadPage()
        self.setCollectButoonStyle()  # 设置收藏按钮式样

    # 按Enter键的处理
    def searchTextBoxOnEnter(self):
        if isEmpty(self.__ui.search_textbox.text()):
            return
        else:
            self.searchButtonOnClicked()

    # 搜索按钮功能
    def searchButtonOnClicked(self):
        try:
            location = Location()
            searchName = self.__ui.search_textbox.text()  # 取得搜索框的值
            if searchName is None or searchName.strip() == sys_const.EMPTY:
                QmyMessageBox.error(self, configure.error_page_title, E0003, QmyMessageBox.OK)
                return
            location.name = searchName  # 地区名字
            location.city = searchName  # 城市名字
            if self.search_data != None and len(self.search_data) != 0:
                data = myServices.get_location_info_by_location_id(self.search_data.get(searchName))
            else:
                raise BusinessException(E0001)
            location.id = data.location_id
            location.province = data.adm1
            if not self.isConnect:
                self.mainPage = QmyMainPage()
                self.mainPage.setStyleSheet(FrameStyleSheet.styleSheetTransparent)
                self.__ui.stackedWidget.addWidget(self.mainPage)
                self.setPageStyle()  # 设置式样
                self.__ui.stackedWidget.setCurrentWidget(self.mainPage)
            self.isConnect = True
            self.mainPage.location = location
            self.mainPage.setWeatherDatas()  # 设置天气数据
            self._location = self.mainPage.location
            self.setCollectButoonStyle()  # 设置收藏按钮式样
            self.setBackgroundImage(self.mainPage.weatherInfo)  # 设置背景
        except BusinessException as be:
            self.isConnect = False
            self.setErrorPage()
            logs.error(traceback.format_exc())  # 输出日志信息
            QmyMessageBox.error(self, configure.error_page_title, be.msg, QmyMessageBox.OK)

        except BaseException as e:
            self.isConnect = False
            self.setErrorPage()
            logs.error(traceback.format_exc())
            QmyMessageBox.error(self, configure.error_page_title, str(e), QmyMessageBox.OK)

    # 点击菜单处理
    def itemSelectionOnChanged(self):
        try:
            selection = self.__ui.menu_list.selectedItems()[0].item
            if selection == MenuEnum.collection:
                self.__ui.report_label.setText("收藏")
                self.hideReportTitle()  # 隐藏头部
                collectionPage = QmyCollectedPage()
                collectionPage.itemClicked.connect(self.locationItemOnClicked)
                self.__ui.stackedWidget.addWidget(collectionPage)
                self.__ui.stackedWidget.setCurrentWidget(collectionPage)
                self.__ui.menu_list.setStyleSheet(menuWidgetStyleSheet)
            elif selection == MenuEnum.report:
                self.__ui.report_label.setText("")
                self.__ui.report_label.hide()
                self.showReportTitle()  # 显示头部
                self.reloadPage()  # 刷新页面
                self.setCollectButoonStyle()  # 设置收藏按钮式样

                self.__ui.menu_list.setStyleSheet(menuWidgetStyleSheet)
                # 初始化定时器
                self.timer = QTimer(self)
                self.timer.timeout.connect(self.refreshButtonOnClicked)
                # 启动定时器,设置一小时刷新一次
                self.timer.start(int(timer))
            else:
                self.__ui.report_label.setText("设置")
                self.hideReportTitle()  # 隐藏头部
                settingsPage = setPage()
                # settingsPage.mySignal.connect(self.themeItemOnClicked)
                self.__ui.stackedWidget.addWidget(settingsPage)
                self.__ui.stackedWidget.setCurrentWidget(settingsPage)
                self.__ui.menu_list.setStyleSheet(menuWidgetStyleSheet)
        except BusinessException as be:
            self.setErrorPage()
            QmyMessageBox.error(self, configure.error_page_title, be.msg, QmyMessageBox.OK)

    # 收藏页面点击某个城市的槽函数
    def locationItemOnClicked(self, e):
        self.mainPage.location = e
        try:

            self.mainPage.setWeatherDatas()  # 设置天气数据
        except BusinessException as be:
            self.setErrorPage()
            QmyMessageBox.error(self, configure.error_page_title, be.msg, QmyMessageBox.OK)
        self._location = self.mainPage.location
        self.setCollectButoonStyle()  # 设置收藏按钮式样
        self.__ui.stackedWidget.setCurrentWidget(self.mainPage)
        self.showReportTitle()
        self.__ui.menu_list.setCurrentRow(0)
        self.setBackgroundImage(self.mainPage.weatherInfo)  # 设置背景

    # 隐藏头部
    def hideReportTitle(self):
        self.__ui.collect_btn.hide()
        self.__ui.refresh_btn.hide()
        self.__ui.search_textbox.hide()
        self.__ui.search_btn.hide()
        self.__ui.frame_3.hide()

    # 显示头部
    def showReportTitle(self):
        self.__ui.collect_btn.show()
        self.__ui.refresh_btn.show()
        self.__ui.search_textbox.show()
        self.__ui.search_btn.show()
        self.__ui.frame_3.show()

    # 菜单按钮处理
    def menuButtonOnClicked(self):
        self.menuIsDisplay()

    # 收藏按钮处理
    def collectButtonOnClicked(self):
        try:

            location = self._location
            if myServices.location_collect_exists(location):
                if myServices.get_location_collection_data(location.id).deleted_flag:
                    myServices.get_location_collection_count()
                    result = QmyMessageBox.question(self, configure.question_page_title, Q0001,
                                                    QmyMessageBox.YES | QmyMessageBox.NO)
                    if result == 1:
                        location.deleted_flag = False
                        myServices.upsert_location_collect(location)
                else:
                    result = QmyMessageBox.question(self, configure.question_page_title, Q0002,
                                                    QmyMessageBox.YES | QmyMessageBox.NO)
                    if result == 1:
                        location.deleted_flag = True
                        myServices.upsert_location_collect(location)
            else:
                myServices.get_location_collection_count()
                result = QmyMessageBox.question(self, configure.question_page_title, Q0001,
                                                QmyMessageBox.YES | QmyMessageBox.NO)
                if result == 1:
                    location.deleted_flag = False
                    myServices.upsert_location_collect(location)

            self.setCollectButoonStyle()  # 设置收藏按钮式样
        except BusinessException as be:
            logs.error(traceback.format_exc())  # 输出日志信息
            QmyMessageBox.error(self, configure.error_page_title, be.msg, QmyMessageBox.OK)

        except Exception as e:
            logs.error(traceback.format_exc())
            QmyMessageBox.error(self, configure.error_page_title, e.msg, QmyMessageBox.OK)

    def setErrorPage(self):
        errorPage = QWidget()
        self.setCollectButoonStyle()  # 设置收藏按钮式样
        self.__ui.stackedWidget.addWidget(errorPage)
        self.__ui.stackedWidget.setCurrentWidget(errorPage)

    def onTxtChoosed(self, txt):
        self.__ui.search_textbox.setText(txt)


    @pyqtSlot(str)
    def searchTextOnChanged(self, e):
        if e is None or str(e).strip() == sys_const.EMPTY:
            return
        self.search_data = OrderedDict()
        data = myServices.get_location_info_by_location_name(e)
        self.search_data = data
        if len(data) != 0:
            self.m_model.setStringList(data.keys())

    def reloadPage(self):
        try:
            self.mainPage.setPageStyle()
            self.mainPage.setStyleSheet(FrameStyleSheet.styleSheetTransparent)
            if not self.isConnect:
                self.mainPage = QmyMainPage()
                self.mainPage.setStyleSheet(FrameStyleSheet.styleSheetTransparent)
                self.mainPage.location = get_location_by_ip()
                self.__ui.stackedWidget.addWidget(self.mainPage)
                self.mainPage.setWeatherDatas()  # 设置天气数据
                self.setPageStyle()  # 设置式样
            self.mainPage.setWeatherDatas()
            self._location = self.mainPage.location
            self.isConnect = True
            self.__ui.stackedWidget.setCurrentWidget(self.mainPage)
            self.setPageStyle()
            self.setBackgroundImage(self.mainPage.weatherInfo)  # 设置背景
        except BusinessException as be:
            self.isConnect = False
            self.setErrorPage()
            if be.msg == E0002:
                QmyMessageBox.error(self, configure.error_page_title, E0002, QmyMessageBox.OK)
            else:
                QmyMessageBox.error(self, configure.error_page_title, be.msg, QmyMessageBox.OK)
        except Exception as e:
            self.isConnect = False
            self.setErrorPage()
            QmyMessageBox.error(self, configure.error_page_title, str(e), QmyMessageBox.OK)


class RefreshDataTimer(QThread):
    resignal = pyqtSignal()  # 括号里填写信号传递的参数

    def __init__(self):
        super().__init__()

    def __del__(self):
        self.wait()

    def run(self):
        self.resignal.emit()  # 发射信号
