#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   myListPage.py    
@Contact :   wangxin01@loongson.cn
@License :   (C)Copyright 2021

@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
21-6-9 上午5:07   wangxin    1.0         None
'''

# import lib
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QFrame

from src.const.styleSheet import listWidgetStyleSheet
from src.ui.listPage import Ui_ListPage

# 美化样式表
Stylesheet = """
/*设置左侧选项的最小最大宽度,文字颜色和背景颜色*/
QPushButton {
    padding:5px;
    border-radius:20px;
    color:#ffffff;
    background-color:transparent;
    font-family: \'Microsoft YaHei Ui\';
    font-size: 14pt;
    font-weight: bold;
}
"""

StylesheetEnabled = """

QPushButton:hover {
    border-width: 2px; 
    border-style: solid;
    border-color: #ffffff;
    background:rgba(255, 255, 255,0);
}
QPushButton:pressed {
    color:#ffffff;
    background:rgba(255, 255, 255,90);
}
"""

StylesheetNoEnabled = """
QPushButton {
    color:rgba(255, 255, 255,0);
}
"""


class QmyListPage(QFrame):
    def __init__(self, parent=None):
        super().__init__(parent)  # 调用父类构造函数，创建QWidget窗体
        self.__ui = Ui_ListPage()  # 创建UI对象
        self.__ui.setupUi(self)  # 构造UI

        self.__ui.leftButton.setIcon(QIcon())
        self.__ui.report_list.setStyleSheet(listWidgetStyleSheet)
        self.setStyleSheet(Stylesheet)

    # 设置ListItem
    def setListItem(self, items):
        self.__ui.report_list.clear()
        for item in items:
            self.__ui.report_list.addItem(item)  # 将逐时列表对象添加到列表中
            self.__ui.report_list.setItemWidget(item, item.widget)  # 设置用来显示内容的组件
        self.__ui.report_list.setCurrentRow(0)

    # 右边按钮向右滑动
    def rightButtonOnClicked(self):
        pos = self.__ui.report_list.horizontalScrollBar().value()
        self.__ui.report_list.horizontalScrollBar().setValue(pos + 1)
        self.setButtonEnable()

    # 左边按钮向左滑动
    def leftButtonOnClicked(self):
        pos = self.__ui.report_list.horizontalScrollBar().value()
        self.__ui.report_list.horizontalScrollBar().setValue(pos - 1)
        self.setButtonEnable()

    # 设置按钮可用性
    def setButtonEnable(self):
        pos = self.__ui.report_list.horizontalScrollBar().value()
        maxPos = self.__ui.report_list.horizontalScrollBar().maximum()
        if maxPos == 0:
            self.__ui.leftButton.setEnabled(False)
            self.__ui.rightButton.setEnabled(False)
        else:
            self.__ui.leftButton.setEnabled(True)
            self.__ui.rightButton.setEnabled(True)
            if maxPos == pos:
                self.__ui.rightButton.setEnabled(False)
            elif pos == 0:
                self.__ui.leftButton.setEnabled(False)

        if self.__ui.leftButton.isEnabled():
            self.__ui.leftButton.setStyleSheet(StylesheetEnabled)
        else:
            self.__ui.leftButton.setStyleSheet(StylesheetNoEnabled)

        if self.__ui.rightButton.isEnabled():
            self.__ui.rightButton.setStyleSheet(StylesheetEnabled)
        else:
            self.__ui.rightButton.setStyleSheet(StylesheetNoEnabled)

    def resizeEvent(self, e):
        self.setButtonEnable()

    def showEvent(self, e):
        self.setButtonEnable()