#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   myMainPage.py    
@Contact :   wangxin01@loongson.cn
@License :   (C)Copyright 2021

@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
21-5-31 上午10:20   wangxin    1.0         None
'''

# import lib
from collections import OrderedDict

from PyQt5.QtCore import QThread, pyqtSignal
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QWidget

from src.const.const import sys_const
from src.const.styleSheet import fontColorWhite, FrameStyleSheet, styleSheetTransparent
from src.controller.myListPage import QmyListPage
from src.controller.plot_pyqt import PlotCanvas
from src.model.dailyWeatherInfo import DailyWeatherInfo
from src.model.hourlyWeatherInfo import HourlyWeatherInfo
from src.model.location import Location
from src.module import logHelper
from src.services import myServices
from src.ui.mainPage import Ui_MainPage
from src.ui.myDailyListItem import QmyDailyListItem
from src.ui.myHourlyListItem import QmyHourlyListItem
from src.utils.timeutils import get_current_datetime, str2date, date2str, day2weekly, solar2lunar

logs = logHelper.logger.getLogger(__name__)

styleSheet = """
QWidget {background-color:transparent;}
"""


# 通过ip地址取得当前城市信息API
def get_location_by_ip():
    data = myServices.get_location_by_ip(None)
    location = Location()
    location.ip = data.get("result").get("ip")  # IP地址
    location.lat = data.get("result").get("location").get("lat")  # 纬度
    location.lng = data.get("result").get("location").get("lng")  # 经度
    location.nation = data.get("result").get("ad_info").get("nation")  # 国家地区
    location.province = data.get("result").get("ad_info").get("province")  # 省份
    location.city = data.get("result").get("ad_info").get("city")  # 城市
    location.name = data.get("result").get("ad_info").get("city")  # 城市
    location.district = data.get("result").get("ad_info").get("district")  # 区
    location.id = myServices.get_location_id_by_location_name(location.city)  # 地区ID

    return location


# 设置逐时列表Item数据
def getHourlyListItems(datas):
    hourlyListItems = []
    for data in datas:
        hourlyWeatherInfo = HourlyWeatherInfo()

        # 天气类型图标
        weather_type_image = sys_const.WEATHER_ICON_PATH + data.get("icon") + sys_const.PNG_SUFFIX
        hourlyWeatherInfo.weather_type_image = weather_type_image

        # 温度
        hourlyWeatherInfo.temperature = data.get("temp") + sys_const.TEMPERATURE_MARK_C

        # 天气类型
        hourlyWeatherInfo.weather_type = data.get("text")

        # 风速图标
        hourlyWeatherInfo.wind_speed_icon = sys_const.IMAGE_PATH + "wind_speed_icon.svg"

        # 风速
        hourlyWeatherInfo.wind_speed = data.get("windSpeed") + sys_const.SPACE + sys_const.WIND_SPEED_UNIT

        # 时间
        time = date2str(str2date(data.get("fxTime"), sys_const.UTC_FORMATTER),
                        sys_const.TIME_FORMATTER_HH_MM)
        hourlyWeatherInfo.time = time

        item = QmyHourlyListItem(hourlyWeatherInfo)  # 创建逐时列表对象

        hourlyListItems.append(item)

    return hourlyListItems


# 设置逐日列表Item数据
def getDailyListItems(datas):
    dailyListItems = []
    for data in datas:
        dailyWeatherInfo = DailyWeatherInfo()

        # 时间
        dateTime = str2date(data.get("fxDate"), sys_const.TIME_FORMATTER_YYYY_DD_MM_HYPHEN)
        dateStr = date2str(dateTime, sys_const.TIME_FORMATTER_DD)
        weekStr = day2weekly(dateTime)

        dailyWeatherInfo.datetime = dateStr + sys_const.SPACE + sys_const.BRACKET_LEFT + weekStr + sys_const.BRACKET_RIGHT

        # 天气类型图标
        weather_type_image = sys_const.WEATHER_ICON_PATH + data.get("iconDay") + sys_const.PNG_SUFFIX
        dailyWeatherInfo.weather_type_image = weather_type_image

        # 最高温度
        dailyWeatherInfo.temp_max = data.get("tempMax") + sys_const.TEMPERATURE_MARK_C_THUMB

        # 最低温度
        dailyWeatherInfo.temp_min = data.get("tempMin") + sys_const.TEMPERATURE_MARK_C_THUMB

        item = QmyDailyListItem(dailyWeatherInfo)  # 创建逐日列表对象

        dailyListItems.append(item)

    return dailyListItems


class QmyMainPage(QWidget):
    Refresh = True

    def __init__(self, parent=None):
        super().__init__(parent)  # 调用父类构造函数，创建QWidget窗体
        self.hourly = QmyListPage()  # 用于显示逐时详细天气信息的对象
        self.hourly_summary = PlotCanvas()  # 用于显示逐时摘要天气信息的对象
        self.__ui = Ui_MainPage()  # 创建UI对象
        self.__ui.setupUi(self)  # 构造UI
        self.daily = QmyListPage()  # 用于显示逐日详细天气信息的对象
        self.setPageStyle()  # 设置式样
        self._location = Location()
        # 当前天气情况
        self._weatherInfo = sys_const.EMPTY

    # 设置画面式样
    def setPageStyle(self):
        self.setWindowIcon(QIcon(sys_const.IMAGE_PATH + "weather_logo.svg"))  # 设置应用程序logo
        self.__ui.backgroundframe.setStyleSheet(styleSheet)
        ### 设置字体颜色为白色
        # 位置
        self.__ui.location.setStyleSheet(fontColorWhite)
        # 天气类型
        self.__ui.weather_type.setStyleSheet(fontColorWhite)  # 设置字体颜色为白色

        # 当前时间
        self.__ui.dataTime.setStyleSheet(fontColorWhite)  # 设置字体颜色为白色

        # 气温
        self.__ui.temperature.setStyleSheet(fontColorWhite)  # 设置字体颜色为白色
        # 气温符号
        self.__ui.tempMark.setStyleSheet(fontColorWhite)  # 设置字体颜色为白色
        # 最后更新时间
        self.__ui.last_updatetime.setStyleSheet(fontColorWhite)  # 设置字体颜色为白色
        self.__ui.last_updatetime_label.setStyleSheet(fontColorWhite)  # 设置字体颜色为白色
        # 空气质量指数
        self.__ui.air_quality.setStyleSheet(fontColorWhite)  # 设置字体颜色为白色
        self.__ui.air_quality_label.setStyleSheet(fontColorWhite)  # 设置字体颜色为白色
        # 空气质量级别
        self.__ui.air_category.setStyleSheet(fontColorWhite)  # 设置字体颜色为白色

        # 风情
        self.__ui.wind_speed.setStyleSheet(fontColorWhite)  # 设置字体颜色为白色
        self.__ui.wind_speed_label.setStyleSheet(fontColorWhite)  # 设置字体颜色为白色
        # 湿度
        self.__ui.humidity.setStyleSheet(fontColorWhite)  # 设置字体颜色为白色
        self.__ui.humidity_label.setStyleSheet(fontColorWhite)  # 设置字体颜色为白色
        # 线1
        self.__ui.line.setStyleSheet(fontColorWhite)  # 设置字体颜色为白色
        # 线2
        self.__ui.line_2.setStyleSheet(fontColorWhite)  # 设置字体颜色为白色
        # 污染物
        self.__ui.air_primary.setStyleSheet(fontColorWhite)  # 设置字体颜色为白色
        self.__ui.air_primary_label.setStyleSheet(fontColorWhite)  # 设置字体颜色为白色
        # 7日天气预报Label
        self.__ui.daily_report_label.setStyleSheet(fontColorWhite)  # 设置字体颜色为白色
        # 24小时天气预报Label
        self.__ui.hourly_report_label.setStyleSheet(fontColorWhite)  # 设置字体颜色为白色
        # 详细按钮式样美化
        self.__ui.hourly_details_weather_btn.setStyleSheet(FrameStyleSheet.stylesheetLeftButton)
        # 摘要按钮式样美化
        self.__ui.hourly_summary_weather_btn.setStyleSheet(FrameStyleSheet.stylesheetRightButton)
        # 详细按钮处在按压式样
        if QmyMainPage.Refresh:
            self.__ui.hourly_details_weather_btn.setStyleSheet(FrameStyleSheet.stylesheetLeftButtonClicked)
        else:
            self.__ui.hourly_summary_weather_btn.setStyleSheet(FrameStyleSheet.stylesheetRightButtonClicked)

    # 设置天气信息
    def setWeatherDatas(self):
        self.setCurrentWeatherDatas()
        self.setHourlyWeatherInfo()
        self.setDailyWeatherInfo()

    # 取得天气实况信息
    def setCurrentWeatherDatas(self):
        self.__ui.location.setText(self._location.display_name)  # 设置位置值

        # 当前日期
        day_solar = date2str(get_current_datetime(), sys_const.TIME_FORMATTER_DD_MM_HYPHEN_CHAR)
        day_weekly = day2weekly(get_current_datetime())
        day_lunar = solar2lunar(get_current_datetime())
        dayString = day_solar + sys_const.SPACE + day_weekly + sys_const.SPACE + day_lunar
        self.__ui.dataTime.setText(dayString)

        # 最后更新时间
        nowTime = get_current_datetime().strftime(sys_const.TIME_FORMATTER_HH_MM)  # 当前时间
        self.__ui.last_updatetime.setText(nowTime)

        # 同步设置实时天气数据
        datas = myServices.get_current_weather_data(self.location)  # 取得天气实况信息
        self.setCurrentWeather(datas)

        # 异步设置实时天气数据
        self.currentAirQualityDatas = WeatherDataThread(self, self._location, 'get_current_air_quality_data')  # 异步加载数据
        self.currentAirQualityDatas.signal.connect(self.CurrentAirQualityDatasThreadCallback)
        self.currentAirQualityDatas.start()  # 启动线程

    # 设置逐日天气信息
    def setDailyWeatherInfo(self):
        # 异步设置逐日天气信息
        self.dailyDatas = WeatherDataThread(self, self._location, 'get_daily_weather_data')  # 异步加载数据
        self.dailyDatas.signal.connect(self.DailyWeatherInfoThreadCallback)
        self.dailyDatas.start()  # 启动线程

    # 设置逐时天气信息
    def setHourlyWeatherInfo(self):
        # 异步设置逐时天气信息
        self.hourlyDatas = WeatherDataThread(self, self._location, 'get_hourly_weather_data')  # 异步加载数据
        self.hourlyDatas.signal.connect(self.HourlyWeatherInfoThreadCallback)
        self.hourlyDatas.start()  # 启动线程

    # 设置实时天气数据
    def setCurrentWeather(self, current_weather_datas):
        # 天气图标
        weather_type_image_path = sys_const.WEATHER_ICON_PATH + current_weather_datas.get("now").get(
            "icon") + sys_const.PNG_SUFFIX
        self.__ui.weather_type_image.setStyleSheet(
            "#weather_type_image{border-image:url(%s)}" % weather_type_image_path)

        # 天气类型
        self.__ui.weather_type.setText(current_weather_datas.get("now").get("text"))
        self._weatherInfo = current_weather_datas.get("now").get("text")

        # 气温
        self.__ui.temperature.setText(current_weather_datas.get("now").get("temp"))

        # 风情
        wind_info_str = current_weather_datas.get("now").get("windDir") + " " + current_weather_datas.get("now").get(
            "windScale") + sys_const.CHINESE_LEVEL
        self.__ui.wind_speed.setText(wind_info_str)

        # 湿度
        self.__ui.humidity.setText(current_weather_datas.get("now").get("humidity") + sys_const.PERCENT_MARK)

    # 设置实时天气数据
    def CurrentAirQualityDatasThreadCallback(self, current_air_quality_datas):
        # 空气质量指数
        self.__ui.air_quality.setText(current_air_quality_datas.get("now").get("aqi"))

        # 空气质量级别
        self.__ui.air_category.setText(current_air_quality_datas.get("now").get("category"))

        # 空气污染程度
        self.__ui.air_primary.setText(current_air_quality_datas.get("now").get("primary"))

    def HourlyWeatherInfoThreadCallback(self, hourly_datas):
        self.hourly.setListItem(getHourlyListItems(hourly_datas.get("hourly")))
        plot_dict = OrderedDict()
        for data in hourly_datas.get("hourly"):
            time = date2str(str2date(data.get("fxTime"), sys_const.UTC_FORMATTER),
                            sys_const.TIME_FORMATTER_HH_MM)
            plot_dict[time] = int(data.get('temp'))
        self.hourly_summary.plot(plot_dict)
        self.__ui.stackedWidget.addWidget(self.hourly_summary)
        self.__ui.stackedWidget.addWidget(self.hourly)
        # self.__ui.stackedWidget.setCurrentWidget(self.hourly)
        if QmyMainPage.Refresh:
            self.hourlyDetailsWeatherButtonOnClicked()
        else:
            self.hourlySummaryWeatherButtonOnClicked()

    def DailyWeatherInfoThreadCallback(self, daily_datas):
        # 设置逐日天气信息
        self.daily.setListItem(getDailyListItems(daily_datas.get("daily")))
        self.__ui.dailyBox.addWidget(self.daily)
        self.setPageStyle()  # 设置式样

    def hourlyDetailsWeatherButtonOnClicked(self):
        self.__ui.stackedWidget.setCurrentWidget(self.hourly)
        self.__ui.hourly_details_weather_btn.setStyleSheet(FrameStyleSheet.stylesheetLeftButtonClicked)
        self.__ui.hourly_summary_weather_btn.setStyleSheet(FrameStyleSheet.stylesheetRightButton)
        QmyMainPage.Refresh = True

    def hourlySummaryWeatherButtonOnClicked(self):
        self.__ui.stackedWidget.setCurrentWidget(self.hourly_summary)
        self.__ui.hourly_details_weather_btn.setStyleSheet(FrameStyleSheet.stylesheetLeftButton)
        self.__ui.hourly_summary_weather_btn.setStyleSheet(FrameStyleSheet.stylesheetRightButtonClicked)
        QmyMainPage.Refresh = False

    @property
    def location(self):
        return self._location

    @location.setter
    def location(self, value):
        self._location = value

    @property
    def weatherInfo(self):
        return self._weatherInfo

class WeatherDataThread(QThread):
    signal = pyqtSignal(object)  # 括号里填写信号传递的参数

    def __init__(self, parent, location, functionName):
        super(WeatherDataThread, self).__init__(parent)
        self.location = location
        self.functionName = functionName

    def run(self):
        # 进行任务操作
        try:
            if self.functionName == 'get_current_weather_data':
                datas = myServices.get_current_weather_data(self.location)  # 取得天气实况信息
            elif self.functionName == 'get_hourly_weather_data':
                datas = myServices.get_hourly_weather_data(self.location)  # 取得日天气预报信息
            elif self.functionName == 'get_daily_weather_data':
                datas = myServices.get_daily_weather_data(self.location)  # 取得周天气预报信息
            elif self.functionName == 'get_current_air_quality_data':
                datas = myServices.get_current_air_quality_data(self.location)  # 取得实时空气质量
        except:
            raise
        self.signal.emit(datas)  # 发射信号