#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   myMessageBox.py
@Contact :   wangxin01@loongson.cn
@License :   (C)Copyright 2021

@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
21-5-31 上午5:38   wangxin    1.0         None
'''

from PyQt5.QtCore import Qt, QSize
from PyQt5.QtWidgets import QDialog, QPushButton, QSizePolicy

from src.const.styleSheet import fontColorBlack
from src.lib.messageBox import Ui_MessageBox


class QmyMessageBox(QDialog):
    """ 自定义MessageBox，继承自QDialog
    其中有3个static method可用: information, warning, error，使用方法与QMessageBox基本一致
    Args:
        parent: 父窗口
        title: MessageBox窗口标题 Window Title
        text: 消息文本
        btype: 按钮，有五种类型按钮：YES, NO, CANCLE, CLOSE, OK
        default: 默认按钮，默认值为YES
    """
    # 仅支持这几个按键
    YES = 0x01
    NO = 0x02
    CANCEL = 0x04
    CLOSE = 0x08
    OK = 0x10
    # 返回值
    retValue = 0

    def __init__(self, parent, title: str, text: str, btype=YES, default=YES):
        super().__init__(parent)
        self.setWindowFlags(self.windowFlags() | Qt.FramelessWindowHint)
        self.ui = Ui_MessageBox()
        self.ui.setupUi(self)
        self.setWindowModality(Qt.ApplicationModal)
        self.setAttribute(Qt.WA_QuitOnClose, True)
        self.setWindowTitle(title)
        self.btnList = {}
        self.ui.lb_text.setText(text)
        # 设置字体颜色为黑色
        self.ui.lb_text.setStyleSheet(fontColorBlack)
        self.create_button(btype)
        self.set_default_btn(btype, default)
        self.ui.pbt_icon.hide()
        # 初始化数值为2
        QmyMessageBox.retValue = QmyMessageBox.NO

    def closeButtonOnClicked(self):
        self.accept()

    def create_button(self, btype):
        # 如果按钮是yes
        if btype & QmyMessageBox.YES == QmyMessageBox.YES:
            self.yesbtn = QPushButton(self)
            self.set_button(self.yesbtn, u'是')
            self.btnList[QmyMessageBox.YES] = self.yesbtn
            self.yesbtn.clicked.connect(self.btn_yes)
        # 如果按钮是no
        if btype & QmyMessageBox.NO == QmyMessageBox.NO:
            self.nobtn = QPushButton(self)
            self.set_button(self.nobtn, u'否')
            self.btnList[QmyMessageBox.NO] = self.nobtn
            self.nobtn.clicked.connect(self.btn_no)
        # 如果按钮是cancel
        if btype & QmyMessageBox.CANCEL == QmyMessageBox.CANCEL:
            self.cancelbtn = QPushButton(self)
            self.set_button(self.cancelbtn, u'取消')
            self.btnList[QmyMessageBox.CANCEL] = self.cancelbtn
            self.cancelbtn.clicked.connect(self.btn_cancel)
        # 如果按钮是close
        if btype & QmyMessageBox.CLOSE == QmyMessageBox.CLOSE:
            self.closebtn = QPushButton(self)
            self.set_button(self.closebtn, u'关闭')
            self.btnList[QmyMessageBox.CLOSE] = self.closebtn
            self.closebtn.clicked.connect(self.btn_close)
        # 如果按钮是ok
        if btype & QmyMessageBox.OK == QmyMessageBox.OK:
            self.okbtn = QPushButton(self)
            self.set_button(self.okbtn, u'确认')
            self.btnList[QmyMessageBox.OK] = self.okbtn
            self.okbtn.clicked.connect(self.btn_ok)

    def set_default_btn(self, btype, default):
        # default键必须是一个独立的按键
        if default not in {QmyMessageBox.YES, QmyMessageBox.NO, QmyMessageBox.CANCEL, QmyMessageBox.OK,
                           QmyMessageBox.CLOSE}:
            return
        if default & btype != 0:
            self.btnList[default].setDefault(True)

    def set_button(self, btn: QPushButton, text: str):
        # 设置按钮的一些参数
        btn.setText(text)
        btn.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        btn.setFixedSize(QSize(95, 35))
        self.ui.hlayout.addWidget(btn)

    def btn_yes(self):
        QmyMessageBox.retValue = QmyMessageBox.YES
        self.close()

    def btn_no(self):
        QmyMessageBox.retValue = QmyMessageBox.NO
        self.close()

    def btn_cancel(self):
        QmyMessageBox.retValue = QmyMessageBox.CANCEL
        self.close()

    def btn_close(self):
        QmyMessageBox.retValue = QmyMessageBox.CLOSE
        self.close()

    def btn_ok(self):
        QmyMessageBox.retValue = QmyMessageBox.OK
        self.close()

    def ret_val(self):
        return QmyMessageBox.retValue

    # 使用时调用的静态方法
    @staticmethod
    def information(parent, title, text, btype=YES, default=YES):
        box = QmyMessageBox(parent, title, text, btype, default)
        # box.ui.pbt_icon.setIcon(QIcon(QPixmap("./images/msg/notify.svg")))
        box.exec()
        return box.ret_val()

    @staticmethod
    def warning(parent, title, text, btype=YES, default=YES):
        box = QmyMessageBox(parent, title, text, btype, default)
        # box.ui.pbt_icon.setIcon(QIcon(QPixmap("./images/msg/warning.svg")))
        box.exec()
        return box.ret_val()

    @staticmethod
    def error(parent, title, text, btype=YES, default=YES):
        box = QmyMessageBox(parent, title, text, btype, default)
        # box.ui.pbt_icon.setIcon(QIcon(QPixmap("./images/msg/error.svg")))
        box.exec()
        return box.ret_val()

    @staticmethod
    def question(parent, title, text, btype=YES, default=YES):
        box = QmyMessageBox(parent, title, text, btype, default)
        # box.ui.pbt_icon.setIcon(QIcon(QPixmap("./images/msg/question.svg")))
        box.exec()
        return box.ret_val()
