#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   plot_pyqt.py
@Contact :   wangxin01@loongson.cn
@License :   (C)Copyright 2021

@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
21-6-8 上午5:25   wangxin    1.0         None
'''

# import lib
import sys

import matplotlib
from PyQt5.QtWidgets import QWidget, QVBoxLayout, QApplication

from src.const.const import sys_const

matplotlib.use('Qt5Agg')
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
import matplotlib.pyplot as plt

# plt.rcParams['font.sans-serif'] = ['宋体']  # 用来正常显示中文标签
plt.rcParams['axes.unicode_minus'] = False  # 用来正常显示负号
plt.rcParams['xtick.color'] = "w"
plt.rcParams['ytick.color'] = "w"
plt.rcParams['axes.labelcolor'] = 'w'
plt.rcParams['lines.color'] = 'w'


class PlotCanvas(QWidget):
    def __init__(self, parent=None):
        super(PlotCanvas, self).__init__(parent)

        # CREATE FIGURE AND SETTINGS
        self.figure = plt.figure()
        self.figure.patch.set_facecolor("None")
        self.canvas = FigureCanvas(self.figure)
        self.axes = self.figure.add_subplot(111)

        self.setLayout(QVBoxLayout())
        self.layout().addWidget(self.canvas)

        self.axes = self.figure.add_subplot(1, 1, 1)  # 一行一列第一个子图

    # 绘制折线图
    def plot(self, data_dict):
        self.axes.cla()  # 清空画布
        self.axes.patch.set_alpha(0.)
        self.canvas.setStyleSheet("background-color:transparent;")
        xDatas = []
        yDatas = []

        # for i in range(len(data_dict)):
        #     xDatas.append(data_dict.keys()[i])
        #     yDatas.append(data_dict.values()[i])
        #     x = data_dict.keys()[i]
        #     y = data_dict.values()[i]
        #     if i % 4 == 0:
        #         self.axes.text(x, y, str(y) + sys_const.TEMPERATURE_MARK_C_THUMB, fontsize=10, color="w")

        for x, y in data_dict.items():
            xDatas.append(x)
            yDatas.append(y)

        for i in range(len(xDatas)):
            if i % 4 == 0:
                x = xDatas[i]
                y = yDatas[i]
                self.axes.text(x, y, str(y) + sys_const.TEMPERATURE_MARK_C_THUMB, fontsize=10, color="w")
            if i == len(xDatas)-1:
                x = xDatas[i]
                y = yDatas[i]
                self.axes.text(x, y, str(y) + sys_const.TEMPERATURE_MARK_C_THUMB, fontsize=10, color="w")

        # self.axes.plot(xDatas, yDatas, 'w-o')  # 绘制折线图
        # self.axes.set_ylim(-20, 36)  # 设置纵坐标范围
        self.axes.set_yticks([])  # 关闭纵坐标刻度
        self.axes.spines['left'].set_color('w')
        self.axes.spines['bottom'].set_color('w')
        self.axes.spines['top'].set_color('none')
        self.axes.spines['bottom'].set_color('none')
        self.axes.spines['left'].set_color('none')
        self.axes.spines['right'].set_color('none')
        self.axes.stackplot(xDatas, yDatas, alpha=0.5, color='Azure')
        displayXticks = []
        for x_index in range(0, 24, 4):
            displayXticks.append(x_index)
        displayXticks.append(23)
        self.axes.set_xticks(displayXticks)  #

        self.canvas.draw()


if __name__ == '__main__':
    app = QApplication(sys.argv)
    form = PlotCanvas()
    form.show()
    sys.exit(app.exec_())
