import sys

import yaml
from PyQt5 import QtCore
from PyQt5.QtCore import pyqtSignal
from PyQt5.QtWidgets import QApplication, QMainWindow, QLabel, QRadioButton, QButtonGroup, QComboBox
from yaml import SafeLoader

from src.config.configure import configure_data, currentLocation
from src.const.const import sys_const, conf_const
from src.const.styleSheet import SettingStyleSheet
from src.controller.myCollectedPage import getLocationCollectionItems
from src.ui.settingsPage import Ui_MainWindow


class setPage(QMainWindow):
    # 自定义信号
    mySignal = pyqtSignal()

    def __init__(self, parent=None):
        # self.imageheight = 100
        super().__init__(parent)  # 调用父类构造函数，创建QMainWindow窗体
        self.__ui = Ui_MainWindow()  # 创建UI对象
        self.__ui.setupUi(self)  # 构造UI

        self.setParams()

    def setParams(self):
        self.__ui.label_4.setStyleSheet("#label_4{border-image:url(%s)}" % sys_const.LOGO_PATH)
        self.__ui.label_3.setStyleSheet("#label_3{border-image:url(%s)}" % sys_const.LOGO_PATH)
        self.__ui.label_5.setStyleSheet("#label_5{border-image:url(%s)}" % sys_const.WEATHER_LOGO_PATH)

        self.lb = QLabel("定位设置", self.__ui.tab)
        self.lb.setGeometry(QtCore.QRect(50, 20, 200, 30))
        # 按钮组一，主题设置按钮组
        self.locationSetting = QButtonGroup()
        self.currentLocation = QRadioButton("定位到我的位置", self.__ui.tab)
        self.currentLocation.setGeometry(QtCore.QRect(60, 60, 300, 36))
        self.selectLocation = QRadioButton("默认位置", self.__ui.tab)
        self.selectLocation.setGeometry(QtCore.QRect(60, 95, 300, 36))
        self.combox = QComboBox(self.__ui.tab)
        self.combox.setGeometry(QtCore.QRect(60, 140, 200, 36))
        self.combox.setStyleSheet(SettingStyleSheet.comboxStyleSheet)
        collectedData = getLocationCollectionItems()
        for location in collectedData:
            self.combox.addItem(location.city)

        self.locationSetting.addButton(self.currentLocation)
        self.locationSetting.addButton(self.selectLocation)

        self.locationSetting.buttonClicked.connect(self.LocationSettingOnClicked)
        self.combox.currentIndexChanged[str].connect(self.ComboxChangedEvent)
        with \
                open(sys_const.CONF_PATH, encoding=sys_const.UTF_8) as file_obj:
            setting_parameter_data = yaml.load(file_obj, SafeLoader)
            setting_parameter = setting_parameter_data.get(conf_const.APLLCATION_SETTINGS).get(
                conf_const.SETTING_PARAMETER)

        # strpar = open('./conf/parameter', encoding="utf-8").read()
        if str(setting_parameter) == str(currentLocation):
            self.combox.hide()
            self.currentLocation.setChecked(True)
        else:
            self.combox.show()
            self.selectLocation.setChecked(True)
            self.combox.setCurrentText(str(setting_parameter))

        self.setStyleSheet(SettingStyleSheet.settingStyleSheet)
        self.setDockNestingEnabled(True)
        self.__ui.tabWidget.setAutoFillBackground(True)

    # 下拉框更改事件
    def ComboxChangedEvent(self, str):
        # open("./conf/parameter", 'w', encoding="utf-8").write(str)
        configure_data['apllcation_settings']['setting_parameter'] = str
        with open(sys_const.CONF_PATH, 'w', encoding='utf-8') as f:
            yaml.dump(data=configure_data, stream=f, encoding="utf-8", allow_unicode=True)

    # 主题按钮功能
    def LocationSettingOnClicked(self):
        current = self.currentLocation.isChecked()
        select = self.selectLocation.isChecked()
        if (current):
            self.combox.hide()
            var = currentLocation
        else:
            self.combox.show()
            var = getLocationCollectionItems()[0].city

        # 将本次操作修改记录到配置文件中
        # var = open("./conf/parameter", 'w', encoding="utf-8").write(str(var))
        configure_data['apllcation_settings']['setting_parameter'] = str(var)
        with open(sys_const.CONF_PATH, 'w', encoding='utf-8') as f:
            yaml.dump(data=configure_data, stream=f, encoding="utf-8", allow_unicode=True)
        QApplication.processEvents()
        self.centralWidget().setAutoFillBackground(True)
        self.setAutoFillBackground(True)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    ex = setPage()
    ex.show()
    sys.exit(app.exec_())
