#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   myServices.py
@Contact :   wangxin01@loongson.cn
@License :   (C)Copyright 2021

@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
21-5-27 上午8:52   wangxin    1.0         None
'''

# import lib
import traceback
from collections import OrderedDict

from src.api import locationInfoAPI, weatherInfoAPI
# 根据IP地址获取地区信息
from src.config import configure
from src.const.const import sys_const
from src.dbadapter import orm
from src.exception.BusinessException import BusinessException
from src.message.message import E0002, E0004, E0001
from src.utils.stringUtils import isEmpty


# 根据IP地址取得城市信息
def get_location_by_ip(ip_address):
    try:
        data = locationInfoAPI.get_location_by_ip(ip_address)
        return data
    except:
        raise BusinessException(E0002)


# 取得天气实况信息
def get_current_weather_data(location):
    try:
        if latLngisEmpty(location):  # 经纬度是否为空
            if location.id:
                location_id = location.id
            else:
                location_id = get_location_id_by_location_name(location.city)  # 根据地区ID取得天气信息

            data = weatherInfoAPI.get_current_weather_data(location_id)
        else:
            location_id = str(location.lng) + sys_const.COMMA + str(location.lat)
            data = weatherInfoAPI.get_current_weather_data(location_id)  # 根据经纬度取得天气信息
            if sys_const.RESULT_CODE_200 != data.get("code"):
                if location.id:
                    location_id = location.id
                else:
                    location_id = get_location_id_by_location_name(location.city)  # 根据地区名字查找地区信息
                data = weatherInfoAPI.get_current_weather_data(location_id)
        if data.get("code") != sys_const.RESULT_CODE_200:
            raise BusinessException(E0002)
        return data
    except:
        raise BusinessException(E0002)


# 取得实时空气质量
def get_current_air_quality_data(location):
    if latLngisEmpty(location):  # 经纬度是否为空
        if location.id:
            location_id = location.id
        else:
            location_id = get_location_id_by_location_name(location.city)  # 根据地区名字查找地区信息
        data = weatherInfoAPI.get_current_air_quality_data(location_id)
    else:
        location_id = str(location.lng) + sys_const.COMMA + str(location.lat)
        data = weatherInfoAPI.get_current_air_quality_data(location_id)  # 根据经纬度取得空气质量
        if sys_const.RESULT_CODE_200 != data.get("code"):
            if location.id:
                location_id = location.id
            else:
                location_id = get_location_id_by_location_name(location.city)  # 根据地区名字查找地区信息
            data = weatherInfoAPI.get_current_air_quality_data(location_id)
    if data.get("code") != sys_const.RESULT_CODE_200:
        e = BusinessException(E0002)
        raise e
    return data


# 取得周天气预报信息
def get_daily_weather_data(location):
    if latLngisEmpty(location):  # 经纬度是否为空
        if location.id:
            location_id = location.id
        else:
            location_id = get_location_id_by_location_name(location.city)  # 根据地区名字查找地区信息
        data = weatherInfoAPI.get_daily_weather_data(location_id)
    else:
        location_id = str(location.lng) + sys_const.COMMA + str(location.lat)
        data = weatherInfoAPI.get_daily_weather_data(location_id)  # 根据经纬度取得天气信息
        if sys_const.RESULT_CODE_200 != data.get("code"):
            if location.id:
                location_id = location.id
            else:
                location_id = get_location_id_by_location_name(location.city)  # 根据地区名字查找地区信息
            data = weatherInfoAPI.get_daily_weather_data(location_id)
    if data.get("code") != sys_const.RESULT_CODE_200:
        e = BusinessException(E0002)
        raise e
    return data


# 取得日天气预报信息
def get_hourly_weather_data(location):
    if latLngisEmpty(location):  # 经纬度是否为空
        if location.id:
            location_id = location.id
        else:
            location_id = get_location_id_by_location_name(location.city)  # 根据地区名字查找地区信息
        data = weatherInfoAPI.get_hourly_weather_data(location_id)
    else:
        location_id = str(location.lng) + sys_const.COMMA + str(location.lat)
        data = weatherInfoAPI.get_hourly_weather_data(location_id)  # 根据经纬度取得天气信息
        if sys_const.RESULT_CODE_200 != data.get("code"):
            if location.id:
                location_id = location.id
            else:
                location_id = get_location_id_by_location_name(location.city)  # 根据地区名字查找地区信息
            data = weatherInfoAPI.get_hourly_weather_data(location_id)
    if data.get("code") != sys_const.RESULT_CODE_200:
        e = BusinessException(E0002)
        raise e
    return data


# 根据地区名字查找地区信息
def get_location_id_by_location_name(location_name):
    try:
        if isEmpty(location_name):
            return None
        data = orm.get_location_info_by_location_name(location_name_trim(location_name))  # 从数据库中查询城市信息
        return data
    except:
        raise


# 跟据地区ID查找地区信息
def get_location_info_by_location_id(location_id):
    try:
        return orm.get_location_info_by_location_id(location_id)
    except:
        raise BusinessException(E0001)


# 如果城市名字后面是“市”， 则删除“市”字
def location_name_trim(location_name):
    if not isEmpty(location_name):
        location_name = location_name.rstrip()
        if location_name[-1] == sys_const.CHINESE_CITY:
            location_name = location_name[0:len(location_name) - 1]
    return location_name


# 经纬度是否为空
def latLngisEmpty(location):
    return location.lat is None or location.lng is None


# 取得收藏表数据
def get_location_collection_datas(limitNum):
    return orm.get_location_collection(limitNum)


# 取得收藏表数据个数
def get_location_collection_count():
    count = orm.get_location_collection_count()
    if count >= configure.location_collection_limit_num:
        raise BusinessException(E0004.format(configure.location_collection_limit_num))
    return orm.get_location_collection_count()


# 添加数据到收藏表
def upsert_location_collect(location):
    try:
        orm.upsert_location_collect(location)
    except:
        raise


# 检查该地区是否在收藏表中
def location_collect_exists(location):
    try:
        return orm.location_collect_exists(location.id)
    except:
        raise


# 取得该收藏天气信息
def get_location_collection_data(id):
    return orm.get_location_collection_data(id)


# 一级行政单位去重
def getData(data, str):
    datas = []
    for d in data:
        if str in d.location_name:
            if d.location_name == d.adm2:
                datas.append(d.adm1 + d.adm2 + sys_const.CHINESE_CITY)
            else:
                datas.append(d.adm1 + d.adm2 + sys_const.CHINESE_CITY + d.location_name)
        elif str in d.adm1:
            datas.append(d.adm1)
        elif str in d.adm2:
            datas.append(d.adm1 + d.adm2 + sys_const.CHINESE_CITY)

    return list(set(datas))


# 根据搜索城市取得强相关的城市列表
def get_location_info_by_location_name(name):
    try:

        data = weatherInfoAPI.get_location_info_by_location_name(name)  # 根据搜索城市取得强相关的城市列表
        if data.get("code") != sys_const.RESULT_CODE_200:
            return []
        data_dict = OrderedDict()
        for d in data.get("location"):
            data_dict[d.get("name")] = d.get("id")
        return data_dict
    except:
        traceback.print_exc()
        raise BusinessException(E0002)
