import time
from datetime import timedelta, datetime

# 日历中文索引
from LunarSolarConverter.LunarSolarConverter import LunarSolarConverter, Solar, Lunar

from src.const.const import sys_const

ymc = [u"正", u"二", u"三", u"四", u"五", u"六", u"七", u"八", u"九", u"十", u"十一",u"十二"]
rmc = ["初一", u"初二", u"初三", u"初四", u"初五", u"初六", u"初七", u"初八", u"初九", u"初十", \
       u"十一", u"十二", u"十三", u"十四", u"十五", u"十六", u"十七", u"十八", u"十九", \
       u"二十", u"廿一", u"廿二", u"廿三", u"廿四", u"廿五", u"廿六", u"廿七", u"廿八", u"廿九", u"三十", u"卅一"]

# 日历库实例化
converter = LunarSolarConverter()


# 阳历转阴历
def solar2lunar(day):
    solar = Solar(day.__getattribute__('year'), day.__getattribute__('month'), day.__getattribute__('day'))
    lunar = converter.SolarToLunar(solar)
    return "{0}月{1}".format(ymc[int(lunar.lunarMonth) - 1], rmc[int(lunar.lunarDay) - 1])


# 阴历转阳历
def lunar2solar(day):
    lunar = Lunar(day.__getattribute__('year'), day.__getattribute__('month'), day.__getattribute__('day'))
    solar = converter.LunarToSolar(lunar)
    return "{0}/{1}/{2}".format(solar.solarYear, solar.solarMonth, solar.solarDay)


# 日期转星期
def day2weekly(day):
    dateW = date2str(day, sys_const.TIME_FORMATTER_W)
    weekStr = sys_const.WEEKLY_CHARS
    return sys_const.WEEKLY_CN + weekStr[int(dateW) - 1]


# 取得当前时间
def get_current_datetime():
    return datetime.now()


# 取得日差时间
def date_delta(date, gap):
    delta = timedelta(days=gap)
    n_days = date + delta
    return n_days


# utc转本地时间
def utc2local(utc_dtm):
    # UTC 时间转本地时间（ +8:00 ）
    local_tm = datetime.fromtimestamp(0)
    utc_tm = datetime.utcfromtimestamp(0)
    offset = local_tm - utc_tm
    return utc_dtm + offset


# 本地转utc
def local2utc(local_dtm):
    # 本地时间转 UTC 时间（ -8:00 ）
    return datetime.utcfromtimestamp(local_dtm.timestamp())


# 字符串转时间类型
def str2date(str, date_format="%Y%m%d"):
    date = datetime.strptime(str, date_format)
    return date


# 时间转字符串
def date2str(date, date_formate="%Y%m%d"):
    str = date.strftime(date_formate)
    return str


# 时间转时间戳
def str2timestamp(str, timestamp_len=10):
    date_array = time.strptime(str, "%Y-%m-%d %H:%M:%S")
    timestamp = int(time.mktime(date_array))
    if timestamp_len == 13:
        timestamp *= 1000
    return timestamp


# 时间戳转字符串
def timestamp2date(time_stamp, format="%Y%m%d:%H%M%s"):
    time_stamp = int(time_stamp)
    if len(time_stamp) == 13:
        time_stamp = time_stamp / 1000
    time_array = time.localtime(time_stamp)
    date = time.strftime(format, time_array)
    return date


if __name__ == '__main__':
    nowtime = datetime.now()
